<?php

/**
 * Manages points for the CWL plugins.
 */
class CodupWooLoyaltyPoints 
{
        
    /**
     * Calculates how many points a user should be awarded based on price and point value.
     * @param Float $price
     * @param Float $pointValue defaults to false (gets from WP options)
     * @return integer
     */
    public function calculatePoints($price, $pointValue = false)
    {
        if($pointValue === false){
            $temp = $this->getFieldValuesFromCatAndProductPage(true);
            $pointValue = $this->getPointValue($temp);
        }
        
        if($pointValue == 0){
            return 0;
        }
        
        $pointsEarned = floor($price/$pointValue);
        
        // lets make sure to never give negative points
        return $pointsEarned < 0 ? 0 : $pointsEarned;
    }
    
    /**
     * Credits points to a user.
     * @param int $userId the WP user id
     * @param int $points the number of points to add
     * @param string $date date at which the points were added, used to track expiry, in WP DB date format
     */
    public function creditPoints($userId, $points, $date = null)
    {
        wp_insert_post(array(
            'post_type' => CodupWooLoyalty::TYPE_POINT_RECORD,
            'post_author' => $userId,
            'post_title' => $points,
            'post_status' => 'earned',
            'post_date' => $date
        ));
    }
    
    /**
     * Add a spent points record to the users point history.
     * In essence, reduces total amount of points the user has, while making sure
     * point expirations works correctly.
     * @param int $userId
     * @param int $points how many points to REMOVE (positive number)
     * @param string $date used to track expiry, in WP DB date format
     */
    public function spendPoints($userId, $points, $date = null)
    {
        if($date == null)
        {
            $discountLifespanDays = intval(get_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN)); 
        
            // find the date this should be marked as spent, so point expiry works correctly
            // get all non-expired point records
            $queryArgs = array(
                'numberposts' => -1,
                'author' => $userId,
                'post_type' => CodupWooLoyalty::TYPE_POINT_RECORD,
                'post_status' => array('earned', 'spent'),
                'orderby' => 'date',
                'order' => 'ASC'
            );
            if($discountLifespanDays !== 0)
            {
                $queryArgs['date_query'] = array(
                    'after' => date('Y-m-d', strtotime('-'.$discountLifespanDays.' days'))
                );
            }
            /* @var $pointRecords WC_Post[] */
            $pointRecords = get_posts($queryArgs);
            
            // find the dateat which we have at least the amount of points we are spending
            // so the spent points expire at the same time
            $pointsMatched = 0;
            foreach($pointRecords as $record)
            {
                $pointsMatched += intval($record->post_title);
                if($pointsMatched >= $points)
                {
                    $date = $record->post_date;
                    break;
                }
            }
        }
        
        wp_insert_post(array(
            'post_type' => CodupWooLoyalty::TYPE_POINT_RECORD,
            'post_author' => $userId,
            'post_title' => -1*$points,
            'post_status' => 'spent',
            'post_date' => $date
        ));
    }
    
    /**
     * Get the number of points the user currently has. Does not count expired points.
     * @param int $userId
     * @global wpdb $wpdb
     */
    public function getPoints($userId)
    {
        global $wpdb;
        
        $discountLifespanDays = intval(get_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN)); 
    
        $query = 'SELECT 
                    SUM(post_title) 
                FROM 
                    '.$wpdb->posts.' 
                WHERE 
                    post_author = '.$userId.' AND
                    post_type = "'.CodupWooLoyalty::TYPE_POINT_RECORD.'"';
        
        // $discountLifespanDays 0 means valid forever
        if($discountLifespanDays > 0){
            $query .= ' AND post_date > NOW() - INTERVAL '.$discountLifespanDays.' DAY';
        }
        
        $points = $wpdb->get_var($query);
        
        return $points == null ? 0 : $points;
    }
    
    /**
     * Get the point value setting.
     * @return int
     */
    public function getPointValue($value)
    {
        return floatval(get_option(CodupWooLoyalty::OPT_POINT_VALUE) + $value);  ;
    }
    
    /**
     * Get the point discount value setting.
     * @return float
     */
    public function getMaxPointDiscount($value)
    {
        return floatval(get_option(CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT) + $value);   ;
    }
    
    /**
     * Get the max allowed discount that can be applied using points.
     * @return float
     */
    public function getPointDiscountValue()
    {
        return floatval(get_option(CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE));
    }
    
    /**
     * Saves into to session how many points the user has chosen to spend.
     * @param type $spendingNow
     */
    public function savePointsToSpend($spendingNow)
    {
        WC()->session->set('cwl_spend_points', $spendingNow);
    }
    
    /**
     * Get the number of points the user has chosen to spend.
     * @return int
     */
    public function getPointsToSpend()
    {
        $spendingNow = WC()->session->get('cwl_spend_points', 0);
        if(isset($_POST['cwl_spend_points'])){
            $spendingNow = filter_input(INPUT_POST, 'cwl_spend_points', FILTER_SANITIZE_STRING);
        }
        return $spendingNow;
    }
    
    /**
     * Get the mamximum number of points user can spent for the specified price.
     * @param float $price
     * @param int $userId
     * @return int
     */
    public function getMaxAllowedToSpend($price, $userId)
    {
        
        $temp = $this->getFieldValuesFromCatAndProductPage(false);
        $maxDiscount = $this->getMaxPointDiscount($temp)/100;
        
        // calc how many points would need to be spent for max discount
        $discount = $price*$maxDiscount;
        $maxPoints = floor($discount/$this->getPointDiscountValue());
        
        // check how many points user has
        $userPoints = $this->getPoints($userId);
        
        // return whichever was smaller
        return min($maxPoints, $userPoints);
    }
    
    /**
     * Get the Point Value from Woocmmerce Category Page fields AND Product Page fields.
     * Get the Maximum Point Discount from Woocmmerce Category Page fields AND Product Page fields.
     * @param booelan for returning value whether points value or maximum points discount  
     * @return int
     */
    public function getFieldValuesFromCatAndProductPage($bool = false)
    {
        global $woocommerce;
        $items = $woocommerce->cart;
        $total_point_value = 0;
        $total_max_point_value = 0;

        if($items != NULL)
        {
            foreach( $items->get_cart() as $item => $cart_item )
            {        
                $product_id = $cart_item['product_id'];     // product id of item in order list
                $category_id = wp_get_post_terms($product_id, 'product_cat',array('fields'=>'ids'));    // category id of item in order list
                
                //Get Point Value of Product Fields of each category of respective product in order list
                $product_point_value = get_post_meta($product_id,'wc_settings_tab_codup_woo_loyalty_product_points_value',true);
                
                //Get Point value of Category Fields of each product in order list
                $cat_point_value = get_term_meta($category_id[0],'wc_settings_tab_codup_woo_loyalty_category_points_value',true);

                if($cat_point_value == NULL || $cat_point_value == '')
                {
                    $cat_point_value = 0;
                }
                else if($product_point_value == NULL || $product_point_value == '')
                {
                    $product_point_value = 0;
                }
                $total_point_value += (int)$cat_point_value + (int)$product_point_value;    // Sum of Point Values
                
                //Get Max Point Value of Product Fields of each product in order list
                $product_max_point_value = get_post_meta($product_id,'wc_settings_tab_codup_woo_loyalty_product_max_point_discount',true);
            
                //Get Max Point Value of Category Fields of each category of respective product in order list
                $cat_max_point_value = get_term_meta($category_id[0],'wc_settings_tab_codup_woo_loyalty_category_max_point_discount',true);
                
                if($cat_max_point_value == NULL || $cat_max_point_value == '')
                {
                    $cat_max_point_value = 0;
                }
                else if($product_max_point_value == NULL || $product_max_point_value == '')
                {
                    $product_max_point_value = 0;
                }
                $total_max_point_value += ((int)$cat_max_point_value) + ((int)$product_max_point_value);    // Sum of Max Point Values
            }
            return ($bool) ? $total_point_value : $total_max_point_value ;
        }
    }
    
}