<?php

/**
 * Provides validation for admin panel options for the plugin.
 * 
 * @global string CWL_SLUG defined in codup-woo-loyalty.php, contains a string ID 
 * for this plugin.
 */
class CodupWooLoyaltyValidator 
{    
    /**
     * Makes sure the elected order status for adding points is valid.
     */
    public function validateAddPointsStatus($value)
    {
        $statuses = wc_get_order_statuses();
        if(array_key_exists($value, $statuses))
        {
            return $value;
        }
        
        WC_Admin_Settings::add_error(__('Invalid order status in When to Add Points!', CWL_SLUG));
        return null;
    }
    
    /**
     * Checks if discount tiers have numeric values and do not overlap.
     */
    public function validateDiscountTier($value, $option, $rawValue)
    {

        if($value != NULL)
        {
            // remove deleted rows
            $processedValue = $value;            

            // normalize indexes so no numberi skipped
            $processedValue = array_values($processedValue);
            
            $valid = true;
            // check if tier values are numeric
            foreach($processedValue as $index => $tier)
            {
                $valid = $this->validateNumeric($tier['from'], sprintf(__('Tier %d "%s"', CWL_SLUG), $index+1, 'From')) && $valid;
                $valid = $this->validateNumeric($tier['to'], sprintf(__('Tier %d "%s"', CWL_SLUG), $index+1, 'To')) && $valid;
                $valid = $this->validateNumeric($tier['discount'], sprintf(__('Tier %d "%s"', CWL_SLUG), $index+1, 'Discount %')) && $valid;
            
                if($valid)
                {
                    if($tier['from'] > $tier['to'])
                    {
                        $valid = false;
                        WC_Admin_Settings::add_error(sprintf(__('"Form" must be less than "To" in Tier %d!', CWL_SLUG), $index+1));
                    }
                }
            }
            
            if($valid)
            {
                $currSymbol = get_woocommerce_currency_symbol();
                $overlaps = array();
                // check if any tiers overlap
                foreach($processedValue as $index => $tier)
                {
                    foreach($processedValue as $otherIndex => $otherTier)
                    {
                        $pairId = $index.'-'.$otherIndex;
                        if($index != $otherIndex && !in_array($pairId, $overlaps))
                        {
                            if($tier['from'] >= $otherTier['from'] && $tier['from'] <= $otherTier['to'] ||
                                $tier['to'] >= $otherTier['from'] && $tier['to'] <= $otherTier['to'])
                            {
                                $valid = false;
                                WC_Admin_Settings::add_error(sprintf(
                                    __('Tier %s (%s - %s) overlaps with Tier %s (%s - %s) in the range %s - %s. Please make sure these tiers do not overlap!', CWL_SLUG), 
                                    $otherIndex+1,
                                    $currSymbol.$otherTier['from'],
                                    $currSymbol.$otherTier['to'],
                                    $index+1,
                                    $currSymbol.$tier['from'],
                                    $currSymbol.$tier['to'],
                                    $currSymbol.max($tier['from'], $otherTier['from']),
                                    $currSymbol.min($tier['to'], $otherTier['to'])
                                ));

                                $overlaps[] = $pairId;
                                $overlaps[] = $otherIndex.'-'.$index;
                            }
                        }
                        else if(in_array($pairId, $overlaps))
                        {
                            $valid = false;
                        }
                    }
                }
            }
            
            if($valid)
            {
                // keep discount at max 100
                foreach($processedValue as $key => &$tier)
                {
                    if($tier['discount'] > 100)
                    {
                        $tier['discount'] = 100;
                    }
                }
                
                return $processedValue;
            }
            else
            {
                return null;
            }
        }
    }


    /**
     * Validates the point value option, to make sure it is non-negative and numeric.
     * @param string $value
     * @return string|null
     */
    public function validatePointValue($value)
    {
        if($this->validateNumeric($value, __('Point Value', CWL_SLUG)))
        {
            return $value;
        }
        
        return null;
    }
    
    /**
     * Validates the point discount value option, to make sure it is non-negative and numeric.
     * @param string $value
     * @return string|null
     */
    public function validatePointDiscountValue($value)
    {
        if($this->validateNumeric($value, __('Point Discount Value', CWL_SLUG)))
        {
            return $value;
        }
        
        return null;
    }
    
    /**
     * Validates the point lifespan option, to make sure it is non-negative and numeric.
     * @param string $value
     * @return string|null
     */
    public function validatePointLifespan($value)
    {
        if($this->validateNumeric($value, __('Point Liefespan', CWL_SLUG)))
        {
            return $value;
        }
        
        return null;
    }
    
    /**
     * Validates the maximum point discount, to make sure it is non-negative and numeric, 
     * and forces it to be 100 or less.
     * @param string $value
     * @return string|null
     */
    public function validateMaxPointDiscount($value)
    {
        if(!$this->validateNumeric($value, __('Maximum Point Discount', CWL_SLUG)))
        {
            return null;
        }
        
        // you can't beat 100%!
        if($value > 100)
        {
            $value = 100;
        }
        
        return $value;
    }
    
    /**
     * Validates a value to be numeric and non negative.
     * @param string $value
     * @param string $fieldName the name of the field to use in error messages
     * @return type
     */
    public function validateNumeric($value, $fieldName)
    {
        if(!is_numeric($value))
        {
            WC_Admin_Settings::add_error(sprintf(__('%s must be a number!', CWL_SLUG), $fieldName));
            return false;
        }
        if($value < 0)
        {
            WC_Admin_Settings::add_error(sprintf(__('%s must not be negative!', CWL_SLUG), $fieldName));
            return false;
        }
        
        return true;
    }
}