<?php

/**
 * Hooks into WP and WC to provide a loyalty system.
 * 
 * Globals defined in codup-woo-loyalty.php:
 * @global string CWL_SLUG contains a string ID for this plugin.
 * @global string CWL_PLUGIN_DIR_URL the url to this plugins directory
 * @global string CWL_VERSION the plugins version number
 */
class CodupWooLoyalty 
{
    /* Setting value keys */
    const OPT_DISCOUNT_MODE = 'wc_settings_tab_'.CWL_SLUG.'_discount_mode';
    const OPT_POINT_VALUE = 'wc_settings_tab_'.CWL_SLUG.'_points_value';
    const OPT_POINT_DISCOUNT_VALUE = 'wc_settings_tab_'.CWL_SLUG.'_points_discount_value';
    const OPT_DISCOUNT_LIFESPAN = 'wc_settings_tab_'.CWL_SLUG.'_point_lifespan';
    const OPT_MAX_POINT_DISCOUNT = 'wc_settings_tab_'.CWL_SLUG.'_max_point_discount';
    const OPT_ADD_POINTS_STATUS = 'wc_settings_tab_'.CWL_SLUG.'_add_points_status';
    const OPT_DISCOUNT_TIERS = 'wc_settings_tab_'.CWL_SLUG.'_discount_tiers';
    
    const OPT_CATEGORY_MAX_POINT_DISCOUNT = 'wc_settings_tab_'.CWL_SLUG.'_category_max_point_discount';     //_________SPIRINT 2 :support category specific discount_______________________//
    const OPT_CATEGORY_POINTS_VALUE = 'wc_settings_tab_'.CWL_SLUG.'_category_points_value';     //_________SPIRINT 2 :support category specific discount_______________________//
    const OPT_CATEGORY_DISCOUNT_TIERS = 'wc_settings_tab_'.CWL_SLUG.'_category_discount_tiers';     //_________SPIRINT 2 :support category specific discount_______________________//

    const OPT_PRODUCT_MAX_POINT_DISCOUNT = 'wc_settings_tab_'.CWL_SLUG.'_product_max_point_discount';     //_________SPIRINT 3 :support product specific discount_______________________//
    const OPT_PRODUCT_POINTS_VALUE = 'wc_settings_tab_'.CWL_SLUG.'_product_points_value';     //_________SPIRINT 3 :support product specific discount_______________________//
    const OPT_PRODUCT_DISCOUNT_TIERS = 'wc_settings_tab_'.CWL_SLUG.'_product_discount_tiers';     //_________SPIRINT 3 :support product specific discount_______________________//
    
    /* Mode keys */
    const MODE_POINTS = 'points';
    const MODE_TIERS = 'tiers';
    
    /* Other constants */
    const TYPE_POINT_RECORD = 'cwl_point_record';
    
    /**
     * @var CodupWooLoyaltyPoints
     */
    private $pointsManager = null;
    
    /**
     * @var CodupWooLoyaltyTiers
     */
    private $tierManager = null;
    
    public function __construct() 
    {
        $this->pointsManager = new CodupWooLoyaltyPoints();
        $this->tierManager = new CodupWooLoyaltyTiers();
    }

    /**
     * Hooks into WP to create new section in WooCommerce settings.
     */
    public function hookIntoWP()
    {
        $mode = get_option(self::OPT_DISCOUNT_MODE, self::MODE_POINTS);
        
        // filters 
        if($mode == self::MODE_POINTS)
        {
            add_filter('woocommerce_checkout_before_order_review', array($this, 'addEarnedPointsText'));
        }
        add_filter('woocommerce_checkout_login_message', array($this, 'replaceLoginMessage'));
        
        // WP actions
        add_action('init', array($this, 'createPointRecordPostType'));
        
        // WC actions
        add_filter('woocommerce_cart_calculate_fees', array($this, 'applyDiscount'));
        if($mode == self::MODE_POINTS)
        {
            add_action('woocommerce_order_status_changed', array($this, 'creditPoints'), 10, 4);
            add_action('woocommerce_cart_contents', array($this, 'addPointSpendingField'));
            add_action('woocommerce_review_order_after_cart_contents', array($this, 'addPointSpendingField'));
            add_action('woocommerce_add_to_cart', array($this, 'resetPointsToSpend'), 10, 6);
            add_filter('woocommerce_cart_emptied', array($this, 'removePoints'));
        }
        
        // add JS and CSS
        add_action('wp_enqueue_scripts', array($this, 'enqueueScriptsAndStyles'));
    }
    
    /**
     * Credits point to a user account.
     * @param int $id
     * @param string $from order status changed from
     * @param string $to order status changed to
     * @param WC_Order $order
     */
    public function creditPoints($id, $from, $to, $order)
    {
        $orderStatusToReactTo = get_option(self::OPT_ADD_POINTS_STATUS);
        $mode = get_option(self::OPT_DISCOUNT_MODE);
    
        // since woocomerce sometimes doesn't use the wc- prefx
        if($mode == self::MODE_POINTS && ($to == $orderStatusToReactTo || $to == str_replace('wc-', '', $orderStatusToReactTo)))
        {
            $total = $order->get_total();

            $points = $this->pointsManager->calculatePoints($total);

            $this->pointsManager->creditPoints($order->get_user_id(), $points);
        }
    }
    
    /**
     * Registers the CWL Point Record custom post type that keeps track of 
     * point earnings.
     */
    public function createPointRecordPostType()
    {
        register_post_type(self::TYPE_POINT_RECORD, array(
            'capability_type' => array('cwl_point', 'cwl_points'),
            'supports' => false,
            'query_var' => false,
            'delete_with_user' => true,
            'show_in_rest' => true
        ));
    }
    
    /**
     * Add a text field that shows how much the user will earn.
     * @global WooCommerce $woocomerce
     */
    public function addEarnedPointsText()
    {
        global $woocommerce;
        
        $cart = $woocommerce->cart;
        $points = $this->pointsManager->calculatePoints($cart->total);
        
        include('templates/points-earned.php');
    }
    
    /**
     * Adds the input field to select how many points to spend.
     * @param WC_Checkout $checkout
     */
    public function addPointSpendingField($checkout)
    {
        if(is_user_logged_in()){
            $userPoints = $this->pointsManager->getPoints(get_current_user_id());
            $maxAllowed = $this->pointsManager->getMaxAllowedToSpend(WC()->cart->subtotal, get_current_user_id());
            $pointDiscountValue = $this->pointsManager->getPointDiscountValue();

            $spendingNow = $this->pointsManager->getPointsToSpend();;
            $spendingNow = min($spendingNow, $maxAllowed);

            include('templates/spend-points.php');
        }
    }
    
    /**
     * Registers the public page JS and CSS files.
     */
    public function enqueueScriptsAndStyles()
    {
        if(is_cart())
        {
            wp_enqueue_script(CWL_SLUG.'_cart_page', CWL_PLUGIN_DIR_URL.'assets/js/cart-page.js', array('jquery'), CWL_VERSION);
        }
        if(is_checkout())
        {
            wp_enqueue_script(CWL_SLUG.'_checkout_page', CWL_PLUGIN_DIR_URL.'assets/js/checkout-page.js', array('jquery'), CWL_VERSION);
        }
        
        wp_enqueue_style(CWL_SLUG.'_main_css', CWL_PLUGIN_DIR_URL.'assets/css/main.css', array(), CWL_VERSION);
    }
    
    /**
     * Apply loyalty discount.
     * @param WC_Cart $cart
     */
    public function applyDiscount($cart)
    {      
        if(is_user_logged_in())
        {
            $mode = get_option(self::OPT_DISCOUNT_MODE, self::MODE_POINTS);
            
            // handle points based discount
            if($mode == self::MODE_POINTS)
            {
                $spendingNow = $this->pointsManager->getPointsToSpend();
                if($spendingNow > 0)
                {
                    $maxAllowed = $this->pointsManager->getMaxAllowedToSpend($cart->subtotal, get_current_user_id()); 
                    $pointsToSpend = min($maxAllowed, max($spendingNow,0));

                    $cart->add_fee(__('Loyalty discount', CWL_SLUG), -1*$pointsToSpend*$this->pointsManager->getPointDiscountValue(), true, '');
                }
                
                $this->pointsManager->savePointsToSpend($spendingNow);
            }
            else
            { //handle tiers based discount
                $discount = $this->tierManager->getUserTierDiscount(get_current_user_id());
                
                if($discount > 0)
                { 
                    $cart->add_fee(sprintf(__('Loyalty discount (%d%%)', CWL_SLUG), $discount), -1*$cart->subtotal*($discount/100), true, '');
                }
            }
        }
    }
    
    /**
     * Removes the points the user has spent.
     */
    public function removePoints()
    {
        $pointsToSpend = $this->pointsManager->getPointsToSpend();
        if($pointsToSpend > 0)
        {
            $this->pointsManager->spendPoints(get_current_user_id(), $pointsToSpend);
        }
    
        $this->resetPointsToSpend();
    }
    
    /**
     * Resets points to spend to 0.
     */
    public function resetPointsToSpend()
    {
        $this->pointsManager->savePointsToSpend(0);
    }
    
    /**
     * Replaces the WooCommerce "Returning customer?" message.
     * @return string
     */
    public function replaceLoginMessage()
    {
        return __('Log in or create an account to earn loyalty points and get discounts!', CWL_SLUG);
    }
}