<?php

/**
 * Tests generic Codup Woo Loyalty functionality.
 */
class PointsTest extends WC_Unit_Test_Case {
    public function testIfPointsAreRemoved(){
        $user = $this->factory->user->create_and_get();
        $pointsManager = new CodupWooLoyaltyPoints();
        
        $pointsManager->creditPoints($user->ID, 500);
        $pointsManager->spendPoints($user->ID, 350);
        
        $this->assertEquals(150, $pointsManager->getPoints($user->ID), 'Points were not removed from user correctly!');        
    }
    
    public function testIfPointsCalulcateCorrectlyWhenSpentAndExpired(){
        $user = $this->factory->user->create_and_get();
        $default = get_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, 10);
        
        $pointsManager = new CodupWooLoyaltyPoints();
        $pointsManager->creditPoints($user->ID, 300);
        $pointsManager->creditPoints($user->ID, 50, date('Y-m-d H:i:s', time()-5*86400)); // 5 days ago
        $pointsManager->creditPoints($user->ID, 150, date('Y-m-d H:i:s', time()-2*86400)); // 2 days ago
        $this->assertEquals(500, $pointsManager->getPoints($user->ID), 'Point caluclation not working!');
        
        $pointsManager->spendPoints($user->ID, 50);
        $this->assertEquals(450, $pointsManager->getPoints($user->ID), 'Point caluclation not working!');
        
        // lets make sure when 7 days have passed, the spent points are expired and no longer affecting the calc
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, 3);
        $this->assertEquals(450, $pointsManager->getPoints($user->ID), 'Point caluclation not working!');
        
        $pointsManager->spendPoints($user->ID, 100);
        $this->assertEquals(350, $pointsManager->getPoints($user->ID), 'Expired spent points incrrectly affect point calulcation!');
        
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, $default);
    }
    
    public function testCantSpendMoreThanUserHasPoints(){
        $user = $this->factory->user->create_and_get();
        $pointsManager = new CodupWooLoyaltyPoints();
        
        $defaultPointDiscountValue = $pointsManager->getPointDiscountValue();
        $defaultMaxDiscount = $pointsManager->getMaxPointDiscount();
        
        update_option(CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE, 1);
        update_option(CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT, 100);
        
        $pointsManager->creditPoints($user->ID, 500);
        
        $this->assertEquals(500, $pointsManager->getMaxAllowedToSpend(1000, $user->ID), 'Max points to spend is more than user has points!');  
                
        update_option(CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE, $defaultPointDiscountValue);
        update_option(CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT, $defaultMaxDiscount);
    }
    
    public function testCantSpendMoreThanMaxPointDiscount(){
        $user = $this->factory->user->create_and_get();
        $pointsManager = new CodupWooLoyaltyPoints();
        
        $defaultPointDiscountValue = $pointsManager->getPointDiscountValue();
        $defaultMaxDiscount = $pointsManager->getMaxPointDiscount();
        
        update_option(CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE, 1);
         // check if max points calc doesnt go over 10% of the price
        update_option(CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT, 10);
        
        $pointsManager->creditPoints($user->ID, 500);
        
        $this->assertEquals(100, $pointsManager->getMaxAllowedToSpend(1000, $user->ID), 'Max points to spend is more than max discount allowed!');  
        $this->assertEquals(2, $pointsManager->getMaxAllowedToSpend(25, $user->ID), 'Max points to spend is more than max discount allowed!');  
               
        update_option(CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE, $defaultPointDiscountValue);
        update_option(CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT, $defaultMaxDiscount);
    }
    
    public function testIfExpiredPointAreNotCounted(){
        $user = $this->factory->user->create_and_get();
        $default = get_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, 1);
        
        $pointsManager = new CodupWooLoyaltyPoints();
        $pointsManager->creditPoints($user->ID, 100);
        $pointsManager->creditPoints($user->ID, 50, date('Y-m-d H:i:s', time()-2*86400)); // 2 days ago
        
        $this->assertEquals(100, $pointsManager->getPoints($user->ID), 'Point expiration not working!');
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, $default);
    }
    
    public function testIfPointsAddedOnOrderStatusChange(){
        $this->prepOrderTest();
        
        $user = $this->factory->user->create_and_get();
        $order = WC_Helper_Order::create_order($user->ID);
        $order->set_total(125.25);
        $order->set_status('wc-processing');
        $order->save();
        $order->set_status('wc-completed');
        $order->save();
        
        $pointsManager = new CodupWooLoyaltyPoints();
        $this->assertEquals(250, $pointsManager->getPoints($user->ID), 'Invalid point number after purchases!');
    }    
    
    public function testIfPointsNotAddedOnOtherOrderStatusChange(){
        $this->prepOrderTest();
        
        $user = $this->factory->user->create_and_get();
        $order = WC_Helper_Order::create_order($user->ID);
        $order->set_total(125.25);
        $order->set_status('wc-processing');
        $order->save();
        $order->set_status('wc-pending');
        $order->save();
        
        $pointsManager = new CodupWooLoyaltyPoints();
        $this->assertEquals(0, $pointsManager->getPoints($user->ID), 'Added points on a status change when shouldn\'t!');
    }
    
    public function testIfNoPointsAddedInTierMode(){
        $this->prepOrderTest();
        update_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_TIERS);
        
        $user = $this->factory->user->create_and_get();
        $order = WC_Helper_Order::create_order($user->ID);
        $order->set_total(125.25);
        $order->set_status('wc-processing');
        $order->save();
        $order->set_status('wc-completed');
        $order->save();
        
        $pointsManager = new CodupWooLoyaltyPoints();
        $this->assertEquals(0, $pointsManager->getPoints($user->ID), 'Added points in tier mode!');
    }
    
    private function prepOrderTest(){
        update_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_POINTS);
        update_option(CodupWooLoyalty::OPT_ADD_POINTS_STATUS, 'wc-completed');
        update_option(CodupWooLoyalty::OPT_POINT_VALUE, 0.5); // 1 point per $0.50 spent
    }
    
    
    public function testPointsAreCalulcatedCorrectlyFromIntegerPrice(){
        $this->_testCalulcatePoints(1, 100, 100);
    }
    
    public function testPointsAreCalulcatedCorrectlyFromFractionalPrice(){
        $this->_testCalulcatePoints(2, 123.55, 61);
    }
    
    public function testPointsNeverAreNegative(){
        $this->_testCalulcatePoints(2, -123.55, 0);
    }
    
    public function testPointsForCents(){
        $this->_testCalulcatePoints(0.02, 14.23, 711);
    }
    
    public function testNoPointIfLessThanPointValue(){
        $this->_testCalulcatePoints(4.5, 2.1, 0);
    }

    /**
     * Convenience method for testing the private calculatePoints method of CodupWooLoyalty.
     */
    private function _testCalulcatePoints($pointValue, $price, $expected, $message = false){
        $pointsManager = new CodupWooLoyaltyPoints();
        $this->assertEquals(
            $expected, 
            $pointsManager->calculatePoints($price, $pointValue), 
            $message ? $message : 'CodupWooLoyalty::calculatePoints did not correctly calculate points for $'.$price.' (point value: '.$pointValue.')!'
        );      
    }
    
    /**
     * Call protected/private method of a class.
     *
     * @param object &$object    Instantiated object that we will run method on.
     * @param string $methodName Method name to call
     * @param array  $parameters Array of parameters to pass into method.
     *
     * @return mixed Method return.
     */
    public function invokeMethod(&$object, $methodName, array $parameters = array())
    {
        $reflection = new \ReflectionClass(get_class($object));
        $method = $reflection->getMethod($methodName);
        $method->setAccessible(true);

        return $method->invokeArgs($object, $parameters);
    }
}
