<?php

/**
 * Tests the admin form validator.
 */
class ValidatorTest extends WC_Unit_Test_Case {
	/**
	 * A single example test.
	 */
	public function testValidatePointStatus() {
        $validator = new CodupWooLoyaltyValidator();
        
        $statuses = wc_get_order_statuses();
        
        // check if it can validate each status correctly
        foreach($statuses AS $key => $value){
            $this->assertEquals($key, $validator->validateAddPointsStatus($key));
        }
        
        // should fail on fake status
        $this->assertNull($validator->validateAddPointsStatus('cwl_test'));
        
        // lets add it and test again
        add_filter('wc_order_statuses', array($this, 'addNewOrderStatus'));
        $this->assertEquals('cwl_test', $validator->validateAddPointsStatus('cwl_test'));
        remove_filter('wc_order_statuses', array($this, 'addNewOrderStatus'));
        
        // should fail again
        $this->assertNull($validator->validateAddPointsStatus('cwl_test'));
	}
    
    /**
     * Add a new status to WooCommerce.
     */
    public function addNewOrderStatus($statuses){
        $statuses['cwl_test'] = 'CWL TEST STATUS';
        return $statuses;
    }
    
    /**
     * Checks validation for methods that only validate if is non negative and numeric.
     */
    public function testNumericValues(){
        $validator = new CodupWooLoyaltyValidator();
        $methods = array('validatePointValue', 'validatePointDiscountValue', 'validatePointLifespan');
        
        foreach($methods AS $method){
            $failValues = array(-100, 'some random string', -99999999999.9999, -1, -0.000001);
            $validValues = array(0, '100', 99999.99, 1, 0.01);
           
            foreach($failValues AS $value){
                $this->assertNull($validator->$method($value), $method.' failed to invalidate '.$value.'!');
            }
            
            foreach($validValues AS $value){
                $this->assertEquals($value, $validator->$method($value), $method.' failed to validate '.$value.'!');
            }
        }
    }
    
    public function testMaxPointDiscount(){
        $validator = new CodupWooLoyaltyValidator();
        $failValues = array(-100, 'some random string', -99999999999.9999, -1, -0.000001);
        
        foreach($failValues AS $value){
            $this->assertNull($validator->validateMaxPointDiscount($value), 'validateMaxPointDiscount failed to invalidate '.$value.'!');
        }
        
        $validValues = array(0, '100', 1, 0.01);
        
        foreach($validValues AS $value){
            $this->assertEquals($value, $validator->validateMaxPointDiscount($value), 'validateMaxPointDiscount failed to validate '.$value.'!');
        }
        
        // check if > 100 resets to 100
        $this->assertEquals(100, $validator->validateMaxPointDiscount(15236), 'validateMaxPointDiscount failed to reset  15236 to 100!');
    }
    
    /**
     * Tests if discount field validation works correctly.
     */
    public function testValidateDiscountTier(){
        $validator = new CodupWooLoyaltyValidator();
        
        // check if valid tiers are validated correctly
        $discountTiers = array(array(
            'from' => 0,
            'to' => 100,
            'discount' => 1
        ), array(
            'from' => 100.01,
            'to' => 200,
            'discount' => 2
        ), array(
            'from' => 200.01,
            'to' => 300,
            'discount' => 3
        ), array(
            'from' => 300.01,
            'to' => 400,
            'discount' => 4
        ));
        
        // backup orignal array so it can be reued
        $tiersBackup = $discountTiers;
        $size = count($tiersBackup);

        $this->assertEquals($tiersBackup, $validator->validateDiscountTier($discountTiers, null, null));
        
        // check if empty values are removed properly at various places in various combinations
        for($amontToRemove = 1; $amontToRemove <= $size; $amontToRemove++){
            for($i = 0; $i < $size; $i++){
                $discountTiers = $tiersBackup;
                
                for($j = $i; $j < min($i+$amontToRemove, $size); $j++){
                    $discountTiers[$j]['from'] = '';
                    $discountTiers[$j]['to'] = '';
                    $discountTiers[$j]['discount'] = '';
                }

                $removed = min($amontToRemove, $size-$i);
                $this->assertCount(
                    $size-$removed, 
                    $validator->validateDiscountTier($discountTiers, null, null),
                    'Failed to validate discount tiers when '.$removed.' tiers were blank, starting at Tier '.($i+1).'!'
                );
            }
        }
        
        // check if validation fails with wrong data
        $discountTiers = $tiersBackup;
        
        // check if various fail values fail in each field
        $fields = array('from', 'to', 'discount');
        $failValues = array(-100, 'some random string', -99999999999.9999, -1, -0.000001);
        for($i = 0; $i < $size; $i++){
            foreach($fields AS $field){
                foreach($failValues AS $failValue){
                    $discountTiers = $tiersBackup;
                    $discountTiers[$i][$field] = $failValue;
                    
                    $this->assertNull(
                        $validator->validateDiscountTier($discountTiers, null, null),
                        'validateDiscountTier failed to invalidate the value "'.$failValue.'" for field "'.$field.'" for Tier '.$i.'!'
                    );
                }
            }
        }
        
        // check of overlapping fails and from > to fails
        // we'll test 'discount' separatly
        unset($fields[2]);
        for($i = 0; $i < $size; $i++){
            foreach($fields AS $field){
                $discountTiers = $tiersBackup;
                if($i == $size-1){ // test if last one will overlap with previous
                    $discountTiers[$i][$field] = $discountTiers[$i][$field]-201; // force overlap
                }else{
                    $discountTiers[$i][$field] = $discountTiers[$i][$field]+101*($i+1); // force overlap
                }

                $this->assertNull(
                    $validator->validateDiscountTier($discountTiers, null, null),
                    'validateDiscountTier failed to invalidate the overlap for field "'.$field.'" for Tier '.$i.'!'
                );
            }
        }
        
        // check if valid ones don't fail
        $validValues = array(0, '100', 99999.99, 1, 0.01);
        foreach($validValues AS $value){
            $discountTiers = array($tiersBackup[0]);
            $discountTiers[0]['from'] = $value;
            $discountTiers[0]['to'] = $value;

            $this->assertEquals(
                $discountTiers,
                $validator->validateDiscountTier($discountTiers, null, null),
                'validateDiscountTier failed to validate the value "'.$value.'" for field "'.$field.'"!'
            );
        }
        
        // chcck if discount validation works
        $discountTiers = array($tiersBackup[0]);
        $discountTiers[0]['discount'] = 1;
        $this->assertEquals(
            $discountTiers, 
            $validator->validateDiscountTier($discountTiers, null, null),
            'Discount field validation failed!'
        );
        $discountTiers[0]['discount'] = 101;
        $this->assertEquals(
            100, 
            $validator->validateDiscountTier($discountTiers, null, null)[0]['discount'],
            'Discount field validation failed!'
        );
        $discountTiers[0]['discount'] = 9999999999999;
        $this->assertEquals(
            100, 
            $validator->validateDiscountTier($discountTiers, null, null)[0]['discount'],
            'Discount field validation failed!'
        );
    }
}
